# -*- coding:utf-8 -*- 

SERVER = '118.112.186.15'
PORT = 6888
URL = 'http://%s:%s/api/v1/%s'

GET_INDEX = 'GetIndex/%s'

GET_INDEXDATA = 'IndexData/'

GET_HISTORY_BYLENGTH = 'IndexHistory/%s/%s'

GET_INDEXES = 'Indexes/%s'

GET_INDEX_COINS = 'ListIndexCoins/%s'

INDEX_COLS = ['name', 'fullname', 'create_dt', 'type', 'value', 'value_usd', 'percent_change_24h', 'timestamp', 
                     'description', 'description_cn']

HISTORY_COLS = ['date', 'btc_index', 'usdt_index']
MSG_FOR_INPUT = '请至少输入参数index_name'



# -*- coding:utf-8 -*- 
'''
Created on 04/05/2018
@author: Jimmy
'''

import requests
# from .authorize import Authorize
from bbindex.vars import *
from bbindex import base
import pandas as pd

session = requests.Session()

class DataAPI():
# class DataAPI(Authorize):
    def __init__(self, username='', password='', token=''):
        pass
#         Authorize.__init__(self, username, password, token, session)
        ### Authorize
#         self._authorize()
# 
#         if self._isvalid:
#             ### Do smt.
#             pass

    def getIndexes(self, index_name='', index_type=''):
        """
        Get bbindex list
        """
        index_name = '' if index_name is None else index_name
        index_type = '' if index_type is None else index_type
        if (index_name != '') & (index_type==''):
            params = GET_INDEXES%(index_name)
        elif (index_name == '') & (index_type != ''):
            params = GET_INDEXES%(index_type)
        elif (index_name != '') & (index_type != ''):
            params = GET_INDEXES%(index_name)
        else:
            params = GET_INDEXES%('')
        data = base.get_http_result(params)
        if (index_type != ''):
            data = base.to_df(data['data'], index=[0])
        else:
            data = base.to_df(data['data'])
        data = data[INDEX_COLS]
        data['timestamp'] = data['timestamp'].map(lambda x: int2time(x))
        return data
        
        
    def getIndex(self, index_name=''):
        """
        Get bbindex by index_name
        """
        if (index_name=='' or index_name is None):
            print(MSG_FOR_INPUT)
            return None
        params = GET_INDEX%(index_name)
        data = base.get_http_result(params)
        index = Index(index_name)
        ds = data['data']
        df = base.to_df(ds['comps'])
        index.comps = df
        index.name = ds['name']
        index.percent_change_seasonly_btc = ds['percent_change_seasonly_btc']
        index.value_usd= ds['value_usd']
        index.percent_change_24h_usd = ds['percent_change_24h_usd']
        index.percent_change_168h_btc = ds['percent_change_168h_btc']
        index.percent_change_168h_usd = ds['percent_change_168h_usd']
        index.description_cn = ds['description_cn']
        index.percent_change_monthly_usd = ds['percent_change_monthly_usd']
        index.percent_change_seasonly_usd = ds['percent_change_seasonly_usd']
        index.percent_change_monthly_btc = ds['percent_change_monthly_btc']
        index.percent_change_24h_btc = ds['percent_change_24h_btc']
        index.type = ds['type']
        index.value_btc = ds['value_btc']
        index.description = ds['description']
        return index
    
    
    def get_comps(self, index_name=''):
        if (index_name=='' or index_name is None):
            print(MSG_FOR_INPUT)
            return None
        data = self.getIndex(index_name)
        return data.comps
    
    
    def indexHistory(self, index_name='', length=''):
        if length is None:
            length = ''
        if (index_name=='' or index_name is None) & (length == '' or length is None):
            print(MSG_FOR_INPUT)
            return None
        params = GET_HISTORY_BYLENGTH%(index_name , length)
        data = base.get_http_result(params)
        df = pd.DataFrame(data['data']['vs'], columns=HISTORY_COLS)
        df['date'] = df['date'].map(lambda x: int2time(x))
        return df
    
    
    def listIndxCoins(self, index_name=''):
        """
        查询指数所包含的coin数据 :
        param:
        index_name 指数名称 :
        return 
                    返回指数中包含的代币数据数组
        """
        if index_name is None or index_name == '':
            print(MSG_FOR_INPUT)
            return None
        params = GET_INDEX_COINS%(index_name)
        data = base.get_http_result(params)
        data = base.to_df(data['data'])
        return data


class Index(object):
    def __init__(self, name):
        self.name = name

        
def int2time(timestamp):
    import time
    value = time.localtime(timestamp)
    dt = time.strftime('%Y-%m-%d %H:%M:%S', value)
    return dt




# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from __future__ import print_function

import requests
import traceback
import os


class Authorize(object):
    """
        权限认证
    """
    def __init__(self, username='', password='', token='', session=None):
        self.__username = username
        self.__password = password
        self.__token = token
        self.session = session


    def _authorize(self):
        """账户验证
        """
        username, password, token = self.__username, self.__password, \
                                                  self.__token
        if username and password:
            self._isvalid, token = self.__authorize_user(username, password)
            #预留
        elif token:
            self._isvalid = self.__is_token_valid(token)
            #预留



    def __get_permanent_token_and_set_to_cookie(self, token='', cookies={}):
        if not token:
            ret_json = requests.post('', data={'grant_type':'permanent'}, cookies=cookies).json()
            token = ret_json.get('access_token')

        self.__set_token_to_cookie(token)

        return token


    def __set_token_to_cookie(self, token):
        os.environ['dataapi_token'] = token
        cookie_dict = {'cloud-sso-token': token}
        self.session.cookies = requests.utils.cookiejar_from_dict(cookie_dict)
        return token


    def __authorize_user(self, user, pwd):

        ### 2 user type
        data_type = dict(username=user, password=pwd)

        def user_type(data=None):
            res = self.session.post('AUTHORIZE_URL', data)

            if not res.ok or not res.json().get('content', {}).get('accountId', 0):
                return False, None
            else:
                result = res.json()
                token = result.get('content', {}).get('token', {}).get('tokenString', '')
                principal_name = result.get('content', {}).get('principalName', '')
                os.environ['DatayesPrincipalName'] = principal_name
                return True, token

        valid, token = user_type(data_type)

        if not valid:
            return False, None
        else:
            os.environ['access_token'] = token
            return True, token


    def __is_token_valid(self, token):
        """
                        检验 token 是否有效
        """
        try:
            r = self.session.get('URL', cookies={'token': token})
            r_json = r.json()

            if type(r_json) == list:
                r = self.session.get('AUTH_URL', cookies={'token': token})
                r_json = r.json()

                return True
            elif type(r_json) == dict and r_json.get('code', 0) == -403:
                print ('token {} 无效或过期'.format(token))
                return False
            else:
                print ('token 验证异常: {}'.format(r.text))
                return False
        except:
            print ('token 验证异常')
            print ('-' * 80)
            traceback.print_exc()
            print ('-' * 80)
            return False












# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import codecs
import os
import bbindex

def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()

long_desc = """

BBIndex
===============

The World ’ s First Well Diversified Crypto Asset Indices. 

Index Description
--------------

B7:
The B7 (BlockChain Top 7 Index) select the top 7 cryptocurrency ranked by the 30-day average market-cap. This is the mega investment in cryptocurrency market.

B20:
The B20 (BlockChain Top 20 Index ) select the top 20 cryptocurrency ranked by the 30-day average market-cap. This portfolio represents the large-cap investment in cryptocurrency market.

BT7:
The BT7 (BlockChain Tokens Top 7 Index) select the top 7 tokens in cryptocurrency market ranked by the 30-day average market-cap. This portfolio invest in the first tier tokens.

BT30:
The BT30 (BlockChain Tokens Top 30 Index) select the top 30 tokens in cryptocurrency market ranked by the 30-day average market-cap. This portfolio invest in the first tier and the second tier tokens.

B50:
The B50 (BlockChain Top 50 Index ) select the top 50 cryptocurrency ranked by the 30-day average market-cap. This portfolio represent the large-cap and mid-cap investment in cryptocurrency market.

B20XX:
The B20XX (BlockChain Top 20 ex Top 7 Index ) select the top 20 cryptocurrency ranked by the 30-day average market-cap and exclude B7 from it. This portfolio remove B7 from the large-cap investment in cryptocurrency market.
B50XX:

The B50XX (BlockChain Top 50 Top 20 Index ) select the top 50 cryptocurrency ranked by the 30-day average market-cap and exclude B20 from it. This portfolio represent the mid-cap investment in cryptocurrency market.

B7X:
The B7X (BlockChain Top 7 ex Bitcoin Index ) select the top 7 cryptocurrency ranked by the 30-day average market-cap and exclude Bitcoin from it. This portfolio removes the Bitcoin from the mega investment.

B20X:
The B20X (BlockChain Top 20 ex Bitcoin Index ) select the top 20 cryptocurrency ranked by the 30-day average market-cap and exclude Bitcoin from it. This portfolio represents the large-cap investment except Bitcoin in cryptocurrency market.

B50X:
The B50X (BlockChain Top 50 ex Bitcoin Index ) select the top 50 cryptocurrency ranked by the 30-day average market-cap and exclude Bitcoin from it. This portfolio represent the large-cap and mid-cap investment in cryptocurrency market except Bitcoin.

BI7:
The BP7 (BlockChain Platform Top 7 Index) select the top 7 platform coins in cryptocurrency market ranked by the 30-day average market-cap. This portfolio invest in the first tier platform coins.

BC7:
The BC7 (BlockChain Currency Top 7 Index) select the top 7 digital currency in cryptocurrency market ranked by the 30-day average market-cap. This portfolio invest in the first tier digital currency.

BI20:
The BP20 (BlockChain Platform Top 20 Index) select the top 20 platform coins in cryptocurrency market ranked by the 30-day average market-cap. This portfolio invest in the first tier and the second tier platform coins.

BI20XX:
The BP20XX(BlockChain Platform Top 20 ex Top 7 Index) select the top 20 platform coins and exclude BP7 from it. This portfolio invest in the second tier platform coins.

BC20:
The BC20 (BlockChain Currency Top 20 Index) select the top 20 digital currency in cryptocurrency market ranked by the 30-day average market-cap. This portfolio invest in the first tier and the second tier digital currency.

BC20XX:
The BC20XX(BlockChain Currency Top 20 ex Top 7 Index) select the top 20 digital currency and exclude BC7 from it. This portfolio invest in the second tier digital currency.

BT30XX:
The BT30XX(BlockChain Tokens Top 30 ex Top 7 Index) select the top 30 tokens and exclude BT7 from it. This portfolio invest in the second tier tokens.

B20C:
The B20C(BlockChain Top 20 Currency Index) select the digital currency in top 20 cryptocurrency ranked by the 30-day average market-cap. This portfolio invest in the large cap digital currency.

B20T:
The B20T(BlockChain Top 20 Token Index) select the tokens in top 20 cryptocurrency ranked by the 30-day average market-cap. This portfolio invest in the large cap tokens.

B20I:
The B20P(BlockChain Top 20 Platform Index) select the platform coins in top 20 cryptocurrency ranked by the 30-day average market-cap. This portfolio invest in the large cap platform coins.

B50C:
The B50C(BlockChain Top 50 Currency Index) select the digital currency in top 50 cryptocurrency ranked by the 30-day average market-cap. This portfolio invest in the large-cap and mid-cap digital currency.

B50T:
The B50T(BlockChain Top 50 Token Index) select the tokens in top 50 cryptocurrency ranked by the 30-day average market-cap. This portfolio invest in the large-cap and mid-cap tokens.

B50I:
The B50P(BlockChain Top 50 Platform Index) select the platform coins in top 50 cryptocurrency ranked by the 30-day average market-cap. This portfolio invest in the large-cap and mid-cap platform coins.

B50XXC:
The B50XXC(BlockChain Top 50 ex top 20 Currency Index) select the digital currency in top 50 cryptocurrency ranked by the 30-day average market-cap and exclude B20C from it. This portfolio invest in the mid-cap digital currency.

B50XXT:
The B50XXT(BlockChain Top 50 ex top 20 Token Index) select the tokens in top 50 cryptocurrency ranked by the 30-day average market-cap and exclude B20T from it. This portfolio invest in the mid-cap tokens.

B50XXI:
The B50XXP(BlockChain Top 50 ex top 20 Platform Index) select the platform coins in top 50 cryptocurrency ranked by the 30-day average market-cap and exclude B20P from it. This portfolio invest in the mid-cap platform coins.


Install
--------------

    pip install bbindex
    
Upgrade
---------------

    pip install bbindex --upgrade
    
    
"""


setup(
    name='bbindex',
    version=bbindex.__version__,
    description='The World\'s First Well Diversified Crypto Asset Indices',
#     long_description=read("READM.rst"),
    long_description = long_desc,
    author='tushare.org',
    author_email='waditu@163.com',
    license='Apache 2.0',
    url='http://tushare.org',
    keywords='Crypto Asset Index',
    classifiers=['Development Status :: 4 - Beta',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'License :: OSI Approved :: BSD License'],
    packages=find_packages(),
    package_data={'': ['*.csv']},
)